using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using Object = UnityEngine.Object;

namespace CheesyFX
{
    public class FlexRegionSupportTracker
    {
        private FlexRegion flexRegion;
        private List<FlexCollisionTrigger> triggers;
        public HashSet<Rigidbody> stuffSupporting = new HashSet<Rigidbody>();
        private bool _supported;
        public bool colliding;
        public Rigidbody exclude;

        public Action SetReference;
        private Vector3 origin;
        private float down;
        
        
        public bool supported
        {
            get { return _supported; }
            set
            {
                if (_supported == value) return;
                _supported = flexRegion.supported = value;
            }
        }
        private float onStayTimeout;

        public FlexRegionSupportTracker(FlexRegion flexRegion, Rigidbody rigidbody, List<FlexCollisionTrigger> triggers, Rigidbody exclude = null)
        {
            this.flexRegion = flexRegion;
            this.triggers = triggers;
            if (exclude != null) this.exclude = exclude;
            foreach (var trigger in triggers)
            {
                trigger.tracker = this;
            }

            if (flexRegion.id == 0)
            {
                // JSONStorableBool heelColliding;
                // Transform heelCollider;
                if (rigidbody.name.StartsWith("l"))
                {
                    SetReference = () =>
                    {
                        if (FlexIt.lHeelColliding.val)
                        {
                            var hc = FlexIt.lHeelCollider;
                            var up = hc.up;
                            origin = hc.position + .5f * hc.localScale.y * up;
                            down = -up.y;
                        }
                        else if (FlexIt.lToeColliding.val)
                        {
                            var hc = FlexIt.lToeCollider;
                            var up = hc.up;
                            origin = hc.position + .5f * hc.localScale.y * up;
                            down = -up.y;
                        }
                        else
                        {
                            var transform = rigidbody.transform;
                            origin = transform.position;
                            down = transform.forward.y;
                        }
                    };
                }
                else
                {
                    SetReference = () =>
                    {
                        // $"heel: {FlexIt.rHeelColliding.val}".Print();
                        // $"toe: {FlexIt.rToeColliding.val}".Print();
                        if (FlexIt.rHeelColliding.val)
                        {
                            var collider = FlexIt.rHeelCollider;
                            var up = collider.up;
                            origin = collider.position + .5f * collider.localScale.y * up;
                            down = -up.y;
                        }
                        else if (FlexIt.rToeColliding.val)
                        {
                            var collider = FlexIt.rToeCollider;
                            var up = collider.up;
                            origin = collider.position + .5f * collider.localScale.y * up;
                            down = -up.y;
                        }
                        else
                        {
                            var transform = rigidbody.transform;
                            origin = transform.position;
                            down = transform.forward.y;
                        }
                    };
                }
            }
            else SetReference = () =>
            {
                var transform = rigidbody.transform;
                origin = transform.position;
                down = -transform.up.y;
            };
        }
        
        public void RegisterCollision(Collision collision)
        {
            if (collision.rigidbody == exclude) return;
            // if(flexRegion.id == 1)
            // {
            //     var torque = Vector3.Cross(collision.contacts[0].point - origin.position,
            //         collision.impulse / Time.fixedDeltaTime);
            //     torque.Print();
            //     flexRegion.morphs[0][1].morphValue = -torque.x;
            // }
            if(!colliding) colliding = true;
            SetReference();
            Vector3 contactVector = (collision.contacts[0].point - origin).normalized;
            // contactVector.Print();
            // down.Print();
            var supporting = contactVector.y < down;
            if(supporting)
            {
                stuffSupporting.Add(collision.rigidbody);
                if(!supported)
                {
                    supported = true;
                    OnStayRoutine().Start();
                }
            }
        }
        
        public void DeregisterCollision(Collision collision){
            stuffSupporting.Remove(collision.rigidbody);
            if(stuffSupporting.Count == 0)
            {
                colliding = false;
                onStayTimeout = .05f;
            }
        }
        
        public IEnumerator OnStayRoutine(){
            while(stuffSupporting.Count > 0 || onStayTimeout > 0f)
            {
                if(stuffSupporting.Count == 0) onStayTimeout -= Time.deltaTime;
                yield return new WaitForSeconds(Time.fixedDeltaTime);
            }
            supported = false;
            // supported.Print();
        }

        public void Destroy()
        {
            for (int i = 0; i < triggers.Count; i++)
            {
                Object.Destroy(triggers[i]);
            }
        }
        
        public void SetActive(bool val)
        {
            for (int i = 0; i < triggers.Count; i++)
            {
                triggers[i].enabled = val;
            }
        }
    }
}